package com.tencent.igame.tip_flutter_image_view;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.gif.GifDrawable;
import com.bumptech.glide.request.FutureTarget;

import java.util.concurrent.ExecutionException;

/**
 * gif
 */
public class GifBitmapProvider implements BitmapProvider {
    private final Context context;
    private final String imageUrl;
    private final int width;
    private final int height;
    private final boolean centerCrop;

    private Bitmap bitmap;

    private FutureTarget<GifDrawable> gifDrawableFutureTarget;

    private GifDrawable gifDrawable;


    public GifBitmapProvider(Context context, String imageUrl, int width, int height, boolean centerCrop) {
        this.context = context;
        this.imageUrl = imageUrl;
        this.width = width;
        this.height = height;
        this.centerCrop = centerCrop;
        init();
    }

    @Override
    public void init() {
        if (centerCrop) {
            gifDrawableFutureTarget = Glide.with(context).asGif().load(this.imageUrl).centerCrop().submit();
        } else {
            gifDrawableFutureTarget = Glide.with(context).asGif().load(this.imageUrl).submit();
        }
        try {
            gifDrawable = gifDrawableFutureTarget.get();
            gifDrawable.setVisible(true, true);
            gifDrawable.startFromFirstFrame();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }
    }

    @Override
    public Bitmap get() {
        if (bitmap == null) {
            return gifDrawable.getFirstFrame();
        } else {
            return bitmap;
        }
    }

    @Override
    public void next() {
        if (gifDrawable.getCurrent() instanceof BitmapDrawable) {
            bitmap = ((BitmapDrawable) gifDrawable.getCurrent()).getBitmap();
        }
    }

    @Override
    public void clean() {
        if (!gifDrawableFutureTarget.isDone()) {
            Glide.with(context).clear(gifDrawableFutureTarget);
        }
        if (gifDrawable != null) {
            gifDrawable.stop();
        }

    }
}
