
#import "TipFlutterImageViewPlugin.h"
#import "OpenGLRender.h"
#import "NormalImageRenderWorker.h"


@interface TipFlutterImageViewPlugin ()

@property (nonatomic, strong) NSMutableDictionary<NSNumber *, OpenGLRender *> *renders;
@property (nonatomic, strong) NSObject<FlutterTextureRegistry> *textures;

@end

@implementation TipFlutterImageViewPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
            methodChannelWithName:@"com.tencent.igame/flutter_image_view"
            binaryMessenger:[registrar messenger]];
    TipFlutterImageViewPlugin* instance = [[TipFlutterImageViewPlugin alloc] initWithTextures:[registrar textures]];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (instancetype)initWithTextures:(NSObject<FlutterTextureRegistry> *)textures {
    self = [super init];
    if (self) {
        _renders = [[NSMutableDictionary alloc] init];
        _textures = textures;
    }
    return self;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"create" isEqualToString:call.method]) {
        CGFloat width = [call.arguments[@"width"] floatValue];
        CGFloat height = [call.arguments[@"height"] floatValue];

        NSInteger __block textureId;
        id<FlutterTextureRegistry> __weak registry = self.textures;
        NormalImageRenderWorker *worker = [[NormalImageRenderWorker alloc] initWithCall:call];

        OpenGLRender *render = [[OpenGLRender alloc] initWithSize:CGSizeMake(width, height)
                                                     worker:worker
                                                     onNewFrame:^{
                                                         [registry textureFrameAvailable:textureId];
                                                     }];

        textureId = (NSInteger) [self.textures registerTexture:render];
        self.renders[@(textureId)] = render;
        result(@(textureId));

    } else if ([@"dispose" isEqualToString:call.method]) {
        NSNumber *textureId = call.arguments[@"textureId"];
        OpenGLRender *render = self.renders[textureId];
        [render dispose];
        [self.renders removeObjectForKey:textureId];
        result(nil);

    } else {
        result(FlutterMethodNotImplemented);
    }
}

@end
